"""
Trace les positions des 4 satellites galileens, relativement à Jupiter.
Les échelles en x et y sont en rayon de Jupiter. La taille du cercle de Jupiter est approximativement à cette échelle.
Le plan orbital des satellites a été tourné à l'horizontale (voir ci-dessous)
Auteur : David Alberto (www.astrolabe-science.fr)
"""

import ephem#calculs d'éphémérides astronomiques
import matplotlib.pyplot as plt#graphes
import datetime as dt#gestion des dates
import cmath#nombres complexes

date_debut=dt.datetime(2022, 1, 1)#année, mois, jour,heure,min,secondes
"""
Les heures sont en temps universel TU.
Si on indique la date, sans l'heure, c'est 00h00min00s.
Si on veut une heure précise, on indique année, mois, jour, heures, minutes, secondes.
Certaines positions varient beaucoup en quelques heures.
"""
#date_debut=dt.datetime(2022, 1, 27, 18, 51, 34)#année, mois, jour, heures, minutes, secondes
pas=dt.timedelta(days=1)#modifiable. Possible aussi : hours=2...

plt.rcParams["font.family"] = "Ubuntu"#ou autre police installée sur l'ordinateur

listeSatellites={#couleurs selon les goûts... j'ai renoncé à me rapprocher des couleurs réelles, trop similaires
'Io': 'orange', 
'Europe':'thistle', 
'Ganymède': 'violet', 
'Callisto':'green'
}
G=10#nombre de graphiques (1 par pas de temps)
"""
Le plan des orbites étant incliné, je le redresse en faisant une rotation de tous les satellites
dans le plan de la figure. L'angle a été trouvé par tâtonnements. Il est à modifier selon l'époque choisie.
Je définis une fonction qui, à partir des coordonnées (x,y), calcule les coordonnées après rotation
"""
angle=0.40#angle de rotation du plan des orbites, en radians. Positif : sens direct.

def rotation(x0, y0):#renvoie les coordonnées x, y d'un point après rotation d'angle angle
    z=complex(x0, y0)
    R, theta=cmath.polar(z)
    Z2=cmath.rect(R, theta+angle)
    return Z2.real, Z2.imag

fig, axs=plt.subplots(nrows=G, ncols=1, tight_layout=True)
axs[0].set_title("Positions des satellites galiléens relativement à Jupiter")
fig.set_size_inches(21/2.54, 29.7/2.54)#format A4
for N in range(G):
    ax=axs[N]#itération sur les différents graphes
    ax.set_xlim(-27, 27)
    ax.set_xticks([])
    ax.set_ylim(-2, 5)
    ax.set_yticks([])
    date=date_debut+pas*N
    ax.scatter(0, 0, s=250, c='tan', edgecolor='k', zorder=0)#Jupiter
    for satellite in list(listeSatellites):
        if satellite=='Io':
            coord=ephem.Io(date)
        elif satellite=='Europe':
            coord=ephem.Europa(date)
        elif satellite=='Ganymède':
            coord=ephem.Ganymede(date)
        else:
           coord=ephem.Callisto(date)
        x, y, z=-coord.x, -coord.y, coord.z#signes '-' : voir remarque ci-dessous
        X, Y=rotation(x, y)
        ax.scatter(X, Y, label=satellite, c=listeSatellites[satellite], edgecolor='k', zorder=z)
        ax.text(X, Y+2, satellite[0], ha='center', fontsize=8, zorder=z)#initiale au-dessus
        ax.axhline(0, c='lightgray', zorder=-30)#plan équatorial, en fond
#        Date dans le coin du graphique :
    ax.text(0.99, 0.99, date.strftime(
    '%d %b %Y %H h %M min'
    ), c='firebrick', zorder=-30, fontsize=12, ha='right', va='top', transform=ax.transAxes)
"""
Remarque :
L'algorithme du module Ephem donne une abscisse x positive vers l'Est. J'ai donc pris l'opposé de l'abscisse calculée, pour avoir
l'Est à gauche (J'aurais aussi pu inverser le sens de l'axe x). Idem pour les ordonnées y.
Les positions en z (profondeur) sont prises en compte dans l'ordre d'affichage (zorder), 
ce qui permet de retrouver fidèlement les transits et occultations.
"""

fig.text(0.01, 0.002, "D. Alberto (www.astrolabe-science.fr)", fontsize=6)
fig.savefig("SatellitesGalileens.pdf")
fig.savefig("SatellitesGalileens.png", dpi=200)
#plt.show()
